@@ -100,10 +100,9 @@ check_pid_and_port()
         local busy=0
 
         if [ $lsof_available -ne 0 ]; then
-            port_info=$(lsof -Pnl -i ":$port" 2>/dev/null | \
-                        grep -F '(LISTEN)')
+            port_info=$(lsof -Pnl -i ":$port" 2>/dev/null | grep -F '(LISTEN)')
             echo "$port_info" | \
-            grep -q -E "[[:space:]](\\*|\\[?::\\]?):$port[[:space:]]" && busy=1
+            grep -q -E "[[:space:]]\\[?(\\*|[[:xdigit:]]*(:[[:xdigit:]]*)+)(\\](%[^:]+)?)?:$port[[:space:]]" && busy=1
         else
             local filter='([^[:space:]]+[[:space:]]+){4}[^[:space:]]+'
             if [ $sockstat_available -ne 0 ]; then
@@ -122,7 +121,7 @@ check_pid_and_port()
                     grep -F 'users:(' | grep -o -E "$filter")
             fi
             echo "$port_info" | \
-            grep -q -E "[[:space:]](\\*|\\[?::\\]?):$port\$" && busy=1
+            grep -q -E "[[:space:]]\\[?(\\*|[[:xdigit:]]*(:[[:xdigit:]]*)+)(\\](%[^:]+)?)?:$port\$" && busy=1
         fi
 
         if [ $busy -eq 0 ]; then
@@ -354,7 +353,7 @@ SST_PID="$DATA/wsrep_sst.pid"
 check_round=0
 while check_pid "$SST_PID" 0; do
     wsrep_log_info "Previous SST is not completed, waiting for it to exit"
-    check_round=$(( check_round + 1 ))
+    check_round=$(( check_round+1 ))
     if [ $check_round -eq 20 ]; then
         wsrep_log_error "previous SST script still running."
         exit 114 # EALREADY
@@ -370,7 +369,7 @@ check_round=0
 while check_pid "$STUNNEL_PID" 1 "$STUNNEL_CONF"; do
     wsrep_log_info "Lingering stunnel daemon found at startup," \
                    "waiting for it to exit"
-    check_round=$(( check_round + 1 ))
+    check_round=$(( check_round+1 ))
     if [ $check_round -eq 10 ]; then
         wsrep_log_error "stunnel daemon still running."
         exit 114 # EALREADY
@@ -388,7 +387,7 @@ check_round=0
 while check_pid "$RSYNC_PID" 1 "$RSYNC_CONF"; do
     wsrep_log_info "Lingering rsync daemon found at startup," \
                    "waiting for it to exit"
-    check_round=$(( check_round + 1 ))
+    check_round=$(( check_round+1 ))
     if [ $check_round -eq 10 ]; then
         wsrep_log_error "rsync daemon still running."
         exit 114 # EALREADY
@@ -481,11 +480,7 @@ EOF
                         tar_type=2
                     fi
                     if [ $tar_type -eq 2 ]; then
-                        if [ -n "$BASH_VERSION" ]; then
-                            printf '%s' "$binlog_files" >&2
-                        else
-                            echo "$binlog_files" >&2
-                        fi
+                        echo "$binlog_files" >&2
                     fi
                     if [ $tar_type -ne 0 ]; then
                         # Preparing list of the binlog file names:
@@ -618,11 +613,11 @@ FILTER="-f '- /lost+found'
 
         wsrep_log_info "Transfer of InnoDB data files done"
 
-        # second, we transfer InnoDB log files
+        # second, we transfer the InnoDB log file
         rsync ${STUNNEL:+--rsh="$STUNNEL"} \
               --owner --group --perms --links --specials \
               --ignore-times --inplace --dirs --delete --quiet \
-              $WHOLE_FILE_OPT -f '+ /ib_logfile[0-9]*' \
+              $WHOLE_FILE_OPT -f '+ /ib_logfile0' \
               -f '- **' "$ib_log_dir/" \
               "rsync://$WSREP_SST_OPT_ADDR-log_dir" >&2 || RC=$?
 
@@ -854,16 +849,7 @@ EOF
         # backward-incompatible behavior:
         CN=""
         if [ -n "$SSTCERT" ]; then
-            # find out my Common Name
-            get_openssl
-            if [ -z "$OPENSSL_BINARY" ]; then
-                wsrep_log_error \
-                    'openssl not found but it is required for authentication'
-                exit 42
-            fi
-            CN=$("$OPENSSL_BINARY" x509 -noout -subject -in "$SSTCERT" | \
-                 tr ',' '\n' | grep -F 'CN =' | cut -d '=' -f2 | sed s/^\ // | \
-                 sed s/\ %//)
+            CN=$(openssl_getCN "$SSTCERT")
         fi
         MY_SECRET="$(wsrep_gen_secret)"
         # Add authentication data to address
